@@ -26,6 +26,16 @@ public:
 
 public:
 	GtkWidget *m_pWidget;
+	GtkWidget *m_pToolbarBold;
+	GtkWidget *m_pToolbarItalic;
+	GtkWidget *m_pToolbarUnderline;
+	GtkWidget *m_pToolbarColor;
+	GtkWidget *m_pToolbarAttachment;
+	GtkWidget *m_pToolbarStrike;
+
+protected:
+	GtkWidget* create_main_win ();
+	GtkWidget* create_toolbar ();
 };
 
 #endif // MAINWND_H_
