@@ -14,6 +14,7 @@
 #endif // _MSC_VER > 1000
 
 #include "gui/Dialog.h"
+#include <string>
 
 class PasswordDialog : public Dialog  
 {
@@ -22,7 +23,20 @@ public:
 	virtual ~PasswordDialog();
 
 	void SetLabel(const char *szText);
+	void SetRequireCheck(bool bRequire){ m_bRequireCheck = bRequire; }
+	void Destroy();
+
+	const char *GetPassword();
+
 	virtual void Create();
+
+protected:
+	GtkWidget* create_password_dialog (GtkWidget* parent1 = NULL);
+
+public:
+	bool m_bRequireCheck;	// must enter pasword more than once
+	int	 m_nEntryCount;
+	std::string m_strEntry;	// 1st entry
 };
 
 #endif // PASSWORDDIALOG_H_
