@@ -9,15 +9,27 @@
 #include "TreeView.h"
 #include <string.h>	//strlen
 #include <stdio.h>	//FILE
+#include "config.h"
+#include "lib/LinkInfo.h"
+#include "lib/NoteDocument.h"
 
 void UpdateTextFromScreen();
 void UpdateUndoRedoMenus();
+void ExecuteLink(LinkInfo &link);
+int GetSelectedNodeIdx();
+
+extern NoteDocument g_doc;
 
 //declare signal handlers
 gboolean treeview1_popup_menu_event_handler (GtkWidget *widget, GdkEventButton *event, gpointer user_data);	
 void cell_edited_callback(GtkCellRendererText *cell, gchar *path_string, gchar *new_text, gpointer user_data);
+void cell_edited_start_callback (GtkCellRenderer *renderer, GtkCellEditable *editable, gchar *path, gpointer user_data);
+void cell_edited_cancel_callback (GtkCellRenderer *renderer, gpointer user_data);
+
 void on_tree_row_click (GtkTreeView *treeview, gpointer user_data);
+void on_tree_row_collapsed (GtkTreeView *tree_view, GtkTreeIter *iter, GtkTreePath *path, gpointer user_data);
 gint treeview_keyboard_handler(GtkWidget *widget, GdkEventKey *event);
+gboolean on_tree_button_release(GtkWidget *widget, GdkEventButton *event, gpointer user_data);
 
 void on_dnd_row_deleted  (GtkTreeModel *treemodel, GtkTreePath *arg1, gpointer user_data);
 void on_dnd_row_inserted (GtkTreeModel *treemodel, GtkTreePath *arg1, GtkTreeIter *arg2, gpointer user_data);
@@ -30,6 +42,7 @@ TreeView::TreeView()
 	m_bDND = false;
 	m_nFirstDraggedIdx = -1;
 	m_nFirstDroppedIdx = -1;
+	m_bCellEdited = false;
 }
 
 TreeView::~TreeView()
@@ -46,41 +59,56 @@ void TreeView::Create()
 	//
 	// tree initialization
 	//
-	store = gtk_tree_store_new(2, G_TYPE_STRING, GDK_TYPE_PIXBUF);
+	
+	// data store: title, icon, strikethrough, underline, node color
+	store = gtk_tree_store_new(5, G_TYPE_STRING, GDK_TYPE_PIXBUF, G_TYPE_BOOLEAN, G_TYPE_UINT, GDK_TYPE_COLOR);
 	treeview1 = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));
 	g_object_unref (G_OBJECT (store));  //tree now holds reference
 
 	gtk_tree_view_set_headers_visible((GtkTreeView *)treeview1, FALSE);
 	gtk_tree_view_set_enable_search((GtkTreeView *)treeview1, FALSE); 
 	gtk_widget_show (treeview1);
-	
-	g_signal_connect (treeview1, "button_release_event", GTK_SIGNAL_FUNC (treeview1_popup_menu_event_handler), NULL);
-	g_signal_connect (GTK_OBJECT (treeview1), "cursor_changed",	GTK_SIGNAL_FUNC (on_tree_row_click), NULL);
-	g_signal_connect (GTK_OBJECT (treeview1), "key_press_event", GTK_SIGNAL_FUNC (treeview_keyboard_handler), NULL);
+
+	g_signal_connect (treeview1, "button_release_event", G_CALLBACK (treeview1_popup_menu_event_handler), NULL);
+	g_signal_connect (treeview1, "cursor_changed",	     G_CALLBACK (on_tree_row_click), NULL);
+	g_signal_connect (treeview1, "key_press_event",      G_CALLBACK (treeview_keyboard_handler), NULL);
+	g_signal_connect (treeview1, "button-release-event", G_CALLBACK (on_tree_button_release), this);
+	g_signal_connect (treeview1, "button-press-event",   G_CALLBACK (on_tree_button_release), this);
+	g_signal_connect (treeview1, "row-collapsed",	     G_CALLBACK (on_tree_row_collapsed), this);
 	
 	//pack two renderers in the single column
 	column = gtk_tree_view_column_new();
 
 	renderer = gtk_cell_renderer_pixbuf_new();
 	gtk_tree_view_column_pack_start(column, renderer, FALSE);
-	gtk_tree_view_column_set_attributes(column, renderer, "pixbuf", 1, NULL);
+	gtk_tree_view_column_set_attributes(column, renderer, "pixbuf", STORE_IDX_ICON, NULL);
 
 	renderer = gtk_cell_renderer_text_new();
+	g_object_set(renderer, "underline-set",  TRUE, NULL);
+	g_object_set(renderer, "foreground-set", TRUE, NULL);
+
 	gtk_tree_view_column_pack_start(column, renderer, TRUE);
-	gtk_tree_view_column_set_attributes(column, renderer, "text", 0, NULL);
-	
-	g_signal_connect(renderer, "edited", (GCallback) cell_edited_callback, NULL);
-	
+	gtk_tree_view_column_set_attributes(column, renderer, 
+			"text",           STORE_IDX_TEXT, 
+			"strikethrough",  STORE_IDX_STRIKE, 
+			"underline",      STORE_IDX_UNDERLINE,
+			"foreground-gdk", STORE_IDX_COLOR,
+			NULL);
+
+	g_signal_connect(renderer, "edited", (GCallback) cell_edited_callback, this);
+	g_signal_connect(renderer, "editing-started", (GCallback) cell_edited_start_callback, this);
+	g_signal_connect(renderer, "editing-canceled", (GCallback) cell_edited_cancel_callback, this);
+
 	// add the column to the view
-	gtk_tree_view_append_column (GTK_TREE_VIEW (treeview1), column);
+	gtk_tree_view_append_column (GTK_TREE_VIEW(treeview1), column);
 
 	m_pWidget = treeview1;
 
 	EnableDragAndDrop();
 
 	//signals needed for DND reordering
-	g_signal_connect(store, "row_deleted",	G_CALLBACK(on_dnd_row_deleted),   this);
-	g_signal_connect(store, "row_inserted", G_CALLBACK(on_dnd_row_inserted),  this);
+	g_signal_connect(store, "row_deleted",	  G_CALLBACK(on_dnd_row_deleted),   this);
+	g_signal_connect(store, "row_inserted",   G_CALLBACK(on_dnd_row_inserted),  this);
 	g_signal_connect(treeview1, "drag_begin", G_CALLBACK(on_dnd_begin), this);
 	g_signal_connect(treeview1, "drag_end",   G_CALLBACK(on_dnd_end),   this);
 }
@@ -102,6 +130,7 @@ void TreeView::EditLabel()
 
 	if(gtk_tree_selection_get_selected(treesel, &model, &iter))
 	{
+		//TOFIX use GtkCellRenderer::gtk_cell_renderer_start_editing for simpler code?
 		//trigger cell editing (edit close is handled elsewhere)
 		GtkTreeViewColumn *column1 = gtk_tree_view_get_column(treeview, 0);
 		GList *list1 = gtk_tree_view_column_get_cell_renderers(column1);
@@ -370,28 +399,35 @@ void TreeView::SelectionEnd()
 	{
 		GtkTreePath* path1 = gtk_tree_model_get_path(model, &iter);
 
+		//position on the last sibling on this level (if not already)
+		GtkTreeIter parent;
+		gboolean bParent = gtk_tree_model_iter_parent(model, &parent, &iter);
+		
+		//position to last parent's child
+		int nChildren = gtk_tree_model_iter_n_children (model, (bParent)? &parent : NULL);
+		if(nChildren < 1)
+			return;
+		if(!gtk_tree_model_iter_nth_child(model, &iter, (bParent)? &parent : NULL, nChildren-1))
+			return;
+
 		while(1)
 		{
-			//position on the last sibling on this level
-			while(gtk_tree_model_iter_next(model, &iter))
-			{
-				GtkTreeViewColumn *column1 = gtk_tree_view_get_column(treeview, 0);
-				GtkTreePath* path2 = gtk_tree_model_get_path(model, &iter);
-				gtk_tree_view_set_cursor(treeview, path2, column1, FALSE);	//refresh node selection
-				gtk_tree_path_free(path2);
-			}
-
 			//refresh path
 			gtk_tree_path_free(path1);
 			path1 = gtk_tree_model_get_path(model, &iter);
 
-			//if expanded children exists
+			//if expanded children exist
 			if(  gtk_tree_model_iter_has_child(model, &iter) && 
 				 gtk_tree_view_row_expanded(treeview, path1))
 			{
-				//position to first child
-				gtk_tree_path_down(path1);
-				gtk_tree_model_get_iter(model, &iter, path1);
+				//position to last child
+				int nChildren = gtk_tree_model_iter_n_children (model, &iter);
+				if(nChildren < 1)
+					break;
+				GtkTreeIter iterChild;
+				if(!gtk_tree_model_iter_nth_child(model, &iterChild, &iter, nChildren-1))
+					break;
+				iter = iterChild;
 			}
 			else
 				break;
@@ -547,3 +583,99 @@ void on_dnd_end (GtkWidget *view, GdkDragContext *context, gpointer data)
 	pTree->m_nFirstDraggedIdx = -1;
 	UpdateUndoRedoMenus();
 }
+
+gboolean on_tree_button_release(GtkWidget *widget, GdkEventButton *event, gpointer user_data)
+{
+#ifdef _ZAURUS_BUILD
+	TreeView *pTree = (TreeView *)user_data;
+	GtkTreeModel *model = gtk_tree_view_get_model(GTK_TREE_VIEW(widget));
+
+	static GtkTreeIter iterSelected;
+	static bool bIterSelectedValid = false;
+
+	if(event->button == 1 && event->type == GDK_BUTTON_PRESS)
+	{
+		//skip if already in edit mode
+		if(pTree && pTree->m_bCellEdited){
+			bIterSelectedValid = false;	//reset info
+			return FALSE;
+		}
+
+		//get currently selected item
+		GtkTreeSelection* treesel = gtk_tree_view_get_selection (GTK_TREE_VIEW(widget));
+		GtkTreeIter  child;
+		gtk_tree_selection_get_selected(treesel, &model, &child);
+		iterSelected = child;
+		bIterSelectedValid = true;
+	}
+	else if(event->button == 1 && event->type == GDK_BUTTON_RELEASE)
+	{
+		//get currently selected item
+		GtkTreeSelection* treesel = gtk_tree_view_get_selection (GTK_TREE_VIEW(widget));
+		GtkTreeIter  child;
+		if(!gtk_tree_selection_get_selected(treesel, &model, &child))
+			return FALSE;
+
+		//get tree item that is below mouse pointer
+		int cell_x, cell_y;
+		GtkTreeViewColumn *column = NULL;
+		GtkTreePath *path = NULL;
+		if(gtk_tree_view_get_path_at_pos(GTK_TREE_VIEW(widget),
+										 (int)event->x, (int)event->y, &path, &column, &cell_x, &cell_y))
+		{
+			GtkTreePath *path1, *path2;
+			path1 = gtk_tree_model_get_path(model, &child);
+			path2 = (bIterSelectedValid) ? gtk_tree_model_get_path(model, &iterSelected) : NULL;
+
+			if( path && path1 && path2 &&
+				0 == gtk_tree_path_compare(path, path1) &&
+				0 == gtk_tree_path_compare(path, path2))
+			{
+				//equal paths -> we generated mouse button released event above previously selected item
+
+				//now test if clicked inside of cell area
+				// only text click should count - not expander click or empty area click
+				GdkRectangle rect;
+				gtk_tree_view_get_cell_area(GTK_TREE_VIEW(widget), path, column, &rect);
+
+				if( cell_x > rect.x &&	
+					cell_x < rect.width && 	
+					cell_y > 0 &&
+					cell_y < rect.height)
+				{
+					int nIdx = GetSelectedNodeIdx();
+					if(nIdx >= 0){ //execute link if exists
+						NoteNode &node = g_doc.GetNodeByIdx(nIdx);
+						if(node.m_objLink.IsValid())
+						{
+							ExecuteLink(node.m_objLink);
+
+							gtk_tree_path_free(path);
+							gtk_tree_path_free(path1);
+							gtk_tree_path_free(path2);
+							return FALSE;
+						}
+					}
+
+					pTree->EditLabel();	//trigger label edit mode
+				}
+			}
+			
+			gtk_tree_path_free(path);
+			gtk_tree_path_free(path1);
+			gtk_tree_path_free(path2);
+		}
+	}
+	else 
+#endif
+	if(event->button == 1 && event->type == GDK_2BUTTON_PRESS)
+	{
+		int nIdx = GetSelectedNodeIdx();
+		if(nIdx >= 0){ //execute link if exists
+			NoteNode &node = g_doc.GetNodeByIdx(nIdx);
+			ExecuteLink(node.m_objLink);
+		}
+	}
+
+	return FALSE;
+}
