@@ -22,16 +22,17 @@ class DocumentIterator
 public:
 	DocumentIterator(NoteDocument &doc);
 	virtual ~DocumentIterator();
+	
+	inline NoteDocument &GetDocument(){	return m_doc; }
 
-	int GetNodeCount();
 	int GetChildCount(int nParentID, bool bRecursive = false);
 	int GetChildIdx(int nParentID, int nSiblingIdx);
 
 	bool IsSiblingByIdx(int nIdx1, int nIdx2);
 	bool IsAncestorByIdx(int nAncestorIdx, int nIdx);
+	int CalcNodeLevel(int nIdx);
 
 	//TOFIX move some methods from NoteDocument
-	NoteNode &GetNodeByIdx(int nIdx);
 	NoteNode &GetNodeByID(int nID);
 
 	//recursive index is when we count tree items in depth-first mode
