@@ -39,13 +39,15 @@ public:
 	bool IsOpen() const;
 
 	void Flush();
-	virtual int	 Read(char *szBuffer, int nLen);
-	virtual int	 Write(const char *szBuffer, int nLen);
+	virtual int	Read(char *szBuffer, int nLen);
+	virtual int	Write(const char *szBuffer, int nLen);
 
 	int  WriteString(const char *szBuffer){ return Write(szBuffer, strlen(szBuffer)); };
+	int  ReadLine(char *szBuffer, int nLen);
 
 	INT64 GetPosition() const;
 	INT64 Seek(INT64 offset);
+	INT64 SeekEnd();
 
 protected:
 #ifdef _WIN32
