@@ -16,9 +16,11 @@ std::string GetAppPath();
 
 bool EnsureDirExists(std::string &strDir);
 std::string GetParentDir(const char *szPath);
+std::string GetBaseName(const char *szPath);
 
 void EnsureTerminated(std::string &strPath, char cDelimiter = '/');
 void EnsureNotTerminated(std::string &strPath);
+void EnsureExtension(std::string &strPath, const char *szExt);
 
 #endif  //_FILE_PATH__
 
