@@ -7,6 +7,7 @@
 ////////////////////////////////////////////////////////////////////////////
 
 #include <stdio.h>
+#include <stdlib.h>
 #include <stdarg.h>
 #include <cassert>
 
@@ -46,6 +47,8 @@ void Assert(int exp, const char *fmt, ...)
 
 	#ifdef _WIN32
 		assert(false);	//trigger debugger event
+	#else
+		exit(1);
 	#endif
     }
 }
