@@ -9,8 +9,8 @@
 #ifndef _MRU_H__
 #define _MRU_H__
 
-#ifdef _WIN32
- #pragma warning(disable: 4786)
+#if _MSC_VER > 1000
+  #pragma warning(disable: 4786)
 #endif
 
 #include <string>
@@ -24,7 +24,6 @@ public:
 
 	int  GetFileCount();
 
-	static const char *getfilename();
 	void Load();
 	void Save();
 	void Clear();
